/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.media.impl;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import it.tidalwave.util.As;
import it.tidalwave.util.AsException;
import it.tidalwave.util.Key;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.Parameters;
import it.tidalwave.role.PlainTextRenderable;
import it.tidalwave.role.spi.StringRenderableSupport;
import it.tidalwave.mobile.media.Media;
import lombok.RequiredArgsConstructor;
import static org.openide.util.NbBundle.*;
import static it.tidalwave.mobile.util.Downloadable.Status.*;

/***********************************************************************************************************************
 *
 * @stereotype Role
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor
public class MediaPlainTextRenderable extends StringRenderableSupport implements PlainTextRenderable
  {
    private static final Logger log = LoggerFactory.getLogger(MediaPlainTextRenderable.class);

    private static final Class<MediaPlainTextRenderable> _ = MediaPlainTextRenderable.class; 
    
    @Nonnull
    private final Media media;
    
    @Nonnull
    public String render (final @Nonnull Object... args) 
      {
        final String pattern = (String)args[0];
        final List<Object> objects = new ArrayList<Object>();
        
        for (final Key<?> key : Parameters.find(Key.class, args))
          {
            Object object = null;

            try
              {
                object = media.get(key);

                if (key.equals(Media.RIGHTS))
                  {
                    String licenseName = getMessage(_, "unknownLicense");
                    final String resourceName = "licenseShort-" + object;

                    try
                      {
                        licenseName = getMessage(_, resourceName);
                      }
                    catch (Exception e)
                      {
                        log.error("license not found: {}", resourceName);
                      }
                    
                    objects.add(licenseName);
                  }
                else if (object instanceof As)
                  {
                    objects.add(((As)object).as(Displayable).getDisplayName());
                  }
                else
                  {
                    objects.add(object);
                  }
              }
            catch (AsException e)
              {
                log.error("Not a displayable: {}", object);
                objects.add("unknown");
              }
            catch (NotFoundException e)
              {
                log.warn("{} not found", key);
                objects.add("unknown");
              }
          }

        return String.format(pattern, objects.toArray());
      }    
  }
